==> multiple-file-picker-component-0.0.23.tgz: package/src/multipleFilePicker.tsx <==
import {
  Box,
  Text,
  FormControl,
  Icon,
  Image,
  Progress,
  Flex,
  IconButton,
  Modal,
  useDisclosure,
  ModalOverlay,
  ModalContent,
  ModalBody,
  ModalFooter,
  Button,
} from "@chakra-ui/react";
import { FileError, useDropzone } from "react-dropzone";
import React from "react";
import { Upload, Delete, Movie, FileBlank } from "@feedloop/icon";
import produce from "immer";
import {
  defineComponent,
  RuntimeContext,
  RuntimeInputContext,
} from "@qorebase/app-component";
import { TableSchema, ViewSchema } from "@qorebase/app-runtime/src/interfaces";
import { fallbackImg } from "@qorebase/app-runtime/src/utils/fallbackImage";
import { useHumanize } from "@qorebase/app-runtime/src/utils/humanizeString";
import { ComponentPropertyEditorContainer } from "@qorebase/app-runtime/src/components/ComponentPropertyEditorContainer";
import { useFormSubmitHook } from "@qorebase/app-runtime/src/hooks/useFormSubmitHook";
import { useSource } from "@qorebase/app-runtime/src/hooks/useSource";
import useDisableCondition from "@qorebase/app-runtime/src/hooks/useDisableCondition";
import InputComponentFormLabel from "@qorebase/app-runtime/src/components/InputComponentFormLabel";

export interface RejectedFiles {
  file: File;
  errors: FileError[];
}

type FileMetadata = {
  mimetype: string;
  filename: string;
  size: number;
};

export type CropProps = {
  unit: string;
  height: number;
  aspect: number;
};

export interface FileUpload {
  file: File;
  uploaded: boolean;
}

const MultipleFilePicker = defineComponent({
  name: "Multiple File Picker",
  type: "none",
  icon: "MultipleFiles",
  group: "input",
  display: "inline",
  defaultProps: {
    label: "Multiple File Picker",
    placeholder: "Click or drag file here to upload",
    hidden: false,
    data: {
      inputID: "multiplefilepicker",
      field: { type: "string", options: { format: "text" } },
      validation: {
        required: false,
      },
    },
    fileType: [],
  },
  propDefinition: {
    label: { group: "Design", type: "string", options: { format: "text" } },
    placeholder: {
      group: "Design",
      type: "string",
      options: { format: "text" },
    },
    hidden: {
      group: "Design",
      type: "boolean",
      options: { format: "boolean" },
    },
    data: {
      group: "Design",
      type: "state",
      options: {
        inputID: "multiplefilepicker",
        field: { type: "string", options: { format: "text" } },
      },
    },
    fileType: {
      group: "Design",
      type: "custom",
      options: {
        style: "full-width",
      },
    },
  },
  Editor: {
    fileType: (props) => {
      const column = React.useMemo(
        () => props.values.data.inputID,
        [props.values.data.inputID]
      );
      const columnList = React.useMemo(() => {
        if (
          !!props.view &&
          !!props.view.columns &&
          Array.isArray(props.view.columns)
        ) {
          const view = props.view as TableSchema | ViewSchema;
          return view.columns.filter(
            (c) =>
              c.name === column &&
              (c.type === "file" || c.type === "multiple_files")
          );
        }
        return [];
      }, [props.view, column]);

      const extension = React.useMemo(
        () => (columnList.length ? columnList[0].definition.extensions : []),
        [columnList]
      );
      const title = useHumanize(props.propDef.label ?? props.propName);
      const desc =
        "File type cannot be changed, the data here is based on file extensions column setting type file / multiple files in Qorebase table";

      if (extension.length === 0) return null;

      return (
        <ComponentPropertyEditorContainer title={title} description={desc}>
          <Box py={2}>
            <Text
              fontSize="sm"
              padding={4}
              color="gray.600"
              borderRadius="md"
              backgroundColor="#EDF2F7"
            >
              {extension}
            </Text>
          </Box>
        </ComponentPropertyEditorContainer>
      );
    },
  },
  Component: (props) => {
    const title = props.hooks.useTemplate(props.properties.label);
    const placeholder = props.hooks.useTemplate(props.properties.placeholder);
    const fileSize = 4000000;
    const maxFileSizeText =
      "Maximum File Size: " + `${fileSize / 1000000}` + "MB";
    const [state, setState] = React.useState<{
      files: Record<string, FileUpload>;
      loaded: number;
      total: number;
    }>({ files: {}, loaded: 0, total: 0 });
    const [filesToDelete, setFilesToDelete] = React.useState<FileMetadata[]>(
      []
    );
    const [isDeleteClicked, setIsDeleteClicked] = React.useState(false);

    const client = props.hooks.useClient();
    const source = useSource();

    const tables = RuntimeContext.useSelectState(
      (state) => state.app.schema.tables
    );
    const views = RuntimeContext.useSelectState(
      (state) => state.app.schema.views
    );
    const tableName = React.useMemo(
      () =>
        tables.find((t) => t.name === source.target)?.name ||
        views.find((v) => v.name === source.target)?.query.table,
      [tables, views, source.target]
    );

    const column = props.properties.data.inputID;

    useFormSubmitHook(
      async (row) => {
        if (Object.keys(state.files).length > 0) {
          const uploadToken = await client.project.axios({
            baseURL: client.project.config.endpoint,
            url: `/v1/files/token/table/${tableName}/id/${row.id}/column/${column}?access=write`,
            method: "GET",
          });
          const formData = new FormData();
          for (const file of Object.values(state.files)) {
            formData.append(file.file.name, file.file);
          }
          await client.project.axios.post(
            `/v1/multiple_files/upload?token=${uploadToken.data.token}`,
            formData,
            {
              baseURL: client.project.config.endpoint,
              headers: {
                "Content-Type": "multipart/form-data",
              },
              onUploadProgress: (event: ProgressEvent) => {
                setState((state) =>
                  produce(state, (draft) => {
                    draft.loaded = event.loaded;
                    draft.total = event.total;
                  })
                );
              },
            }
          );
        }
        if (filesToDelete.length > 0) {
          await Promise.all(
            filesToDelete.map(async (file) => {
              const deleteToken = await client.project.axios({
                baseURL: client.project.config.endpoint,
                url: `/v1/files/token/table/${tableName}/id/${row.id}/column/${column}?access=delete`,
                method: "GET",
              });
              return client.project.axios.delete(
                `/v1/files/${file.filename}?token=${deleteToken.data.token}`,
                {
                  baseURL: client.project.config.endpoint,
                }
              );
            })
          );
        }
      },
      [tableName, column, state.files, filesToDelete]
    );
    const getURL = (filename: string) => {
      return `${client.project.config.endpoint}/v1/files/public/${tableName}/${props.data.page.row.id}/${column}/${filename}`;
    };
    const handleFileReader = (file: File) => {
      const reader = new FileReader();
      reader.onload = (e) => {
        Promise.resolve(e.target.result as string).then((value) => {
          setState((prev) => ({ ...prev }));
          file["data"] = value;
        });
      };
      reader.readAsDataURL(file);
    };
    const onDrop = React.useCallback(
      async (acceptedFiles: File[]) => {
        acceptedFiles.forEach(function (x) {
          handleFileReader(x);
        });
        setState((state) =>
          produce(state, (draft) => {
            for (const file of acceptedFiles) {
              draft.files[file.name] = { file, uploaded: false };
            }
          })
        );
      },
      [source, column]
    );

    const { getRootProps, getInputProps } = useDropzone({
      onDrop,
      multiple: true,
    });

    const paddingSpace = {
      paddingTop: 10,
      maxWidth: "100%",
    };

    const uploadedFiles = React.useMemo(
      () =>
        (
          (props.data.page.row[props.properties.data.inputID] ||
            []) as FileMetadata[]
        ).filter(
          (file) =>
            !filesToDelete.find(
              (fileToDelete) => fileToDelete.filename === file.filename
            )
        ),
      [props.data.page.row[props.properties.data.inputID], filesToDelete]
    );
    const filesToUpload = React.useMemo(
      () => Object.values(state.files),
      [state.files]
    );
    const inputActions = RuntimeInputContext.useActions();

    const [validation, setValidation] = React.useState<{
      error?: string;
    }>({ error: undefined });
    React.useEffect(() => {
      if (props.properties.data.validation?.required) {
        if (!uploadedFiles.length && !filesToUpload.length) {
          if (!isDeleteClicked) {
            inputActions.setValidation({
              [props.properties.data.inputID]: {
                error: "",
              },
            });
            setValidation({ error: "" });
          } else {
            inputActions.setValidation({
              [props.properties.data.inputID]: {
                error: "Required field, Please enter the data",
              },
            });
            setValidation({ error: "Required field, Please enter the data" });
          }
        } else if (
          (!!uploadedFiles.length || !!filesToUpload.length) &&
          !!validation.error
        ) {
          inputActions.setValidation({
            [props.properties.data.inputID]: undefined,
          });
          setValidation({});
        }
      } else if (
        !props.properties.data.validation?.required &&
        !!validation.error
      ) {
        inputActions.setValidation({
          [props.properties.data.inputID]: undefined,
        });
        setValidation({});
      }
    }, [
      uploadedFiles,
      filesToUpload,
      validation?.error,
      props.properties.data.validation?.required,
      props.properties.data.inputID,
    ]);
    const { onClose } = useDisclosure();
    const arrSize = Object.keys(state.files).length;
    const currFileSize = Object.values(state.files)[arrSize - 1]?.file.size;
    // @ts-ignore
    const { isDisabled } = useDisableCondition(props.instance);
    return (
      <Box display={props.properties.hidden ? "none" : undefined}>
        <FormControl>
          <InputComponentFormLabel
            title={title}
            name="multiple-file-picker"
            instanceId={props.instance.id}
            fontWeight="normal"
            isRequired={props.properties.data.validation?.required}
            isEmpty={title.length === 0}
          />
          {!!uploadedFiles.length && (
            <Box
              mt={2}
              border="1px"
              borderRadius="md"
              borderColor="surface.900"
              paddingY="5"
              paddingX="3"
            >
              <Box mb={2}>
                <Text fontWeight={"semibold"}>Uploaded files</Text>
              </Box>
              {uploadedFiles.map((file) => (
                <Flex key={file.filename} alignItems="center">
                  <Box
                    display="flex"
                    width="48px"
                    height="48px"
                    bgColor={`${props.app.theme.options.primaryColor}23`}
                    marginRight={5}
                    justifyContent="center"
                    alignItems="center"
                    borderRadius="4px"
                    marginTop={2}
                  >
                    {file.mimetype.startsWith("image") ? (
                      <Image
                        src={getURL(file.filename) ?? fallbackImg}
                        width="48px"
                        height="48px"
                        borderRadius="4px"
                      />
                    ) : file.mimetype.startsWith("video") ? (
                      <Icon
                        as={Movie}
                        boxSize={7}
                        color={`${props.app.theme.options.primaryColor}`}
                      />
                    ) : file.mimetype === "application/pdf" ? (
                      <Icon
                        as={FileBlank}
                        boxSize={7}
                        color={`${props.app.theme.options.primaryColor}`}
                      />
                    ) : (
                      <Icon
                        as={Upload}
                        boxSize={7}
                        color={`${props.app.theme.options.primaryColor}`}
                      />
                    )}
                  </Box>
                  <Box flex={1}>
                    <Text noOfLines={1}>{file.filename}</Text>
                    <Text fontSize="sm" textColor="text.600">
                      {Math.ceil(file.size / 1024)} KB
                    </Text>
                  </Box>
                  <IconButton
                    aria-label="delete file"
                    icon={<Icon as={Delete} />}
                    variant="link"
                    color="#171717"
                    _hover={{ color: "#E03138", bg: "#FCEAEB" }}
                    width="48px"
                    height="48px"
                    onClick={() =>
                      setFilesToDelete((files) => files.concat(file))
                    }
                  />
                </Flex>
              ))}
            </Box>
          )}
          {/* Modal Max upload validation */}
          <Modal
            isOpen={currFileSize > fileSize}
            onClose={onClose}
            isCentered
            size="xs"
          >
            <ModalOverlay />
            <ModalContent paddingY="5">
              <ModalBody>
                <Text textAlign="center" fontSize="xl" fontWeight="bold">
                  Sorry, file size is too big
                </Text>
                <Text textAlign="center" fontSize="sm" color="blackAlpha.500">
                  {maxFileSizeText}
                </Text>
              </ModalBody>
              <ModalFooter>
                <Button
                  onClick={() => setState((prev) => ({ ...prev, files: {} }))}
                  w="full"
                >
                  Ok
                </Button>
              </ModalFooter>
            </ModalContent>
          </Modal>
          {!!filesToUpload.length && (
            <Box
              mt={2}
              border="1px"
              borderRadius="md"
              borderColor="surface.900"
              paddingY="5"
              paddingX="3"
            >
              <Box mb={2}>
                <Text fontWeight={"semibold"}>Files to upload</Text>
              </Box>
              {filesToUpload.map((file) => (
                <Flex key={file.file.name} alignItems="center">
                  <Box
                    display="flex"
                    width="48px"
                    height="48px"
                    bgColor={`${props.app.theme.options.primaryColor}23`}
                    marginRight={5}
                    justifyContent="center"
                    alignItems="center"
                    borderRadius="4px"
                    marginTop={2}
                  >
                    {file.file.type.startsWith("image") ? (
                      <Image
                        src={file.file["data"] ?? fallbackImg}
                        width="48px"
                        height="48px"
                        borderRadius="4px"
                      />
                    ) : file.file.type.startsWith("video") ? (
                      <Icon
                        as={Movie}
                        boxSize={7}
                        color={`${props.app.theme.options.primaryColor}`}
                      />
                    ) : file.file.type === "application/pdf" ? (
                      <Icon
                        as={FileBlank}
                        boxSize={7}
                        color={`${props.app.theme.options.primaryColor}`}
                      />
                    ) : (
                      <Icon
                        as={Upload}
                        boxSize={7}
                        color={`${props.app.theme.options.primaryColor}`}
                      />
                    )}
                  </Box>
                  <Box flex={1}>
                    <Text noOfLines={1}>{file.file.name}</Text>
                    <Text fontSize="sm" textColor="text.600">
                      {Math.ceil(file.file.size / 1024)} KB
                    </Text>
                  </Box>
                  <IconButton
                    aria-label="delete file"
                    icon={<Icon as={Delete} />}
                    variant="link"
                    color="#171717"
                    _hover={{ color: "#E03138", bg: "#FCEAEB" }}
                    width="48px"
                    height="48px"
                    onClick={() => {
                      setState((state) => ({
                        ...state,
                        files: Object.fromEntries(
                          Object.entries(state.files).filter(
                            ([key]) => key !== file.file.name
                          )
                        ),
                      }));
                      setIsDeleteClicked(true);
                    }}
                  />
                </Flex>
              ))}
              {!!state.total && (
                <>
                  <Progress
                    value={Math.round((state.loaded / state.total) * 100)}
                  />
                  <Text fontSize="sm" textColor="blackAlpha.600">
                    {Math.ceil(state.loaded / 1024)}/
                    {Math.ceil(state.total / 1024)} KB
                  </Text>
                </>
              )}
            </Box>
          )}

          <Box
            {...getRootProps({ className: "dropzone" })}
            display="flex"
            flexDirection="column"
            alignItems="center"
            mt={2}
            cursor="pointer"
            border="1px"
            borderRadius="md"
            borderColor="rgb(226, 232, 240)"
            paddingY="10"
            bgColor="surface.900"
            style={{
              border: "1px solid rgb(226, 232, 240)",
              pointerEvents: isDisabled ? "none" : "auto",
              background: isDisabled && "#E7E7E7",
              color: isDisabled && "#A2A2A2",
            }}
          >
            <input
              {...getInputProps()}
              key={props.properties.data.inputID}
              name={props.properties.data.inputID}
              capture={
                props.properties.fileType === "camera"
                  ? "user" || "environment"
                  : undefined
              }
              data-test="file-picker-input"
            />
            <Icon
              as={Upload}
              boxSize={10}
              color={isDisabled ? "#A2A2A2" : "blue.500"}
              data-test="file-picker-input-icon"
            />
            <Text
              style={paddingSpace}
              noOfLines={1}
              align="center"
              w={{ base: "300px", md: "500px" }}
              color={isDisabled && "#A2A2A2"}
            >
              {placeholder}
            </Text>
            <Text fontSize="sm" color="text.600">
              Support for a single and bulk upload.
            </Text>
            <Text fontSize="sm" color="text.600">
              {maxFileSizeText}
            </Text>
          </Box>
        </FormControl>
        {!!validation.error && (
          <Text fontSize="sm" color="red">
            {validation.error}
          </Text>
        )}
      </Box>
    );
  },
});

export default MultipleFilePicker;

==> multiple-file-picker-component-0.0.24.tgz: package/src/multipleFilePicker.tsx <==
import { Box, Text } from "@chakra-ui/react";
import { FileError } from "react-dropzone";
import { defineComponent } from "@qorebase/app-component";
import { TableSchema, ViewSchema } from "@qorebase/app-runtime/src/interfaces";
import React from "react";
import { useHumanize } from "@qorebase/app-runtime/src/utils/humanizeString";
import { ComponentPropertyEditorContainer } from "@qorebase/app-runtime/src/components/ComponentPropertyEditorContainer";
import MultipleFilePickerComponent from "./MultipleFilePickerComponent";

export interface RejectedFiles {
  file: File;
  errors: FileError[];
}

export type CropProps = {
  unit: string;
  height: number;
  aspect: number;
};

export interface FileUpload {
  file: File;
  uploaded: boolean;
}

const MultipleFilePicker = defineComponent({
  name: "Multiple File Picker",
  type: "record",
  icon: "MultipleFiles",
  group: "input",
  display: "inline",
  defaultProps: {
    label: "Multiple File Picker",
    placeholder: "Click or drag file here to upload",
    hidden: false,
    data: {
      inputID: "multiplefilepicker",
      field: { type: "string", options: { format: "text" } },
      validation: {
        required: false,
      },
    },
    fileType: [],
  },
  propDefinition: {
    label: { group: "Design", type: "string", options: { format: "text" } },
    placeholder: {
      group: "Design",
      type: "string",
      options: { format: "text" },
    },
    hidden: {
      group: "Design",
      type: "boolean",
      options: { format: "boolean" },
    },
    data: {
      group: "Design",
      type: "state",
      options: {
        inputID: "multiplefilepicker",
        field: { type: "string", options: { format: "text" } },
      },
    },
    fileType: {
      group: "Design",
      type: "custom",
      options: {
        style: "full-width",
      },
    },
  },
  Editor: {
    fileType: (props) => {
      const column = React.useMemo(
        () => props.values.data.inputID,
        [props.values.data.inputID]
      );
      const columnList = React.useMemo(() => {
        if (
          !!props.view &&
          !!props.view.columns &&
          Array.isArray(props.view.columns)
        ) {
          const view = props.view as TableSchema | ViewSchema;
          return view.columns.filter(
            (c) =>
              c.name === column &&
              (c.type === "file" || c.type === "multiple_files")
          );
        }
        return [];
      }, [props.view, column]);

      const extension = React.useMemo(
        () => (columnList.length ? columnList[0].definition.extensions : []),
        [columnList]
      );
      const title = useHumanize(props.propDef.label ?? props.propName);
      const desc =
        "File type cannot be changed, the data here is based on file extensions column setting type file / multiple files in Qorebase table";

      if (extension.length === 0) return null;

      return (
        <ComponentPropertyEditorContainer title={title} description={desc}>
          <Box py={2}>
            <Text
              fontSize="sm"
              padding={4}
              color="gray.600"
              borderRadius="md"
              backgroundColor="#EDF2F7"
            >
              {extension}
            </Text>
          </Box>
        </ComponentPropertyEditorContainer>
      );
    },
  },
  Component: (props) => <MultipleFilePickerComponent {...props} />,
});

export type MultipleFilePickerComponentType =
  typeof MultipleFilePicker["Component"];

export default MultipleFilePicker;
